#!/usr/bin/python3

import argparse
import os
import re
import subprocess

from contextlib import contextmanager


VIDEO_EXTS = {'mp4', 'avi'}

RE_TRACK = re.compile(r"Track ID (?P<trackid>\d+): (?P<kind>\w+) \((?P<spec>.+)\).*")


@contextmanager
def lang_changer(new_lang):
    """Simple context manager to change the LANG in the environment."""
    old_lang = os.environ['LANG']
    os.environ['LANG'] = new_lang
    yield
    os.environ['LANG'] = old_lang


def _get_tracks(moviefilepath):
    """Get the tracks from the mp4."""
    cmd = ["mkvmerge", "-i", moviefilepath]
    with lang_changer('C'):
        cp = subprocess.run(cmd, stdout=subprocess.PIPE)
        cp.check_returncode()
    lines = cp.stdout.decode("utf8").split("\n")

    tracks = {}
    for line in lines:
        m = RE_TRACK.match(line)
        if m:
            info = m.groupdict()
            tracks.setdefault(info['kind'], []).append(info['trackid'])
    return tracks


def get_output_name(orig_name):
    """Try to infere a name."""
    useful = []
    for part in orig_name.split("."):
        if part.isdigit() and part.startswith("20"):
            break
        useful.append(part)
    return " ".join(useful) + ".mkv"


def main(moviefilepath, srtfilepath, result_filepath, audio_track):
    """Main entry point."""
    tracks = _get_tracks(moviefilepath)

    # select video and audio tracks
    if len(tracks['video']) == 1:
        video_track = tracks['video'][0]
    else:
        print("ERROR: bad video tracks:", tracks['video'])
    if audio_track is not None:
        assert audio_track in tracks['audio']
    elif len(tracks['audio']) == 1:
        audio_track = tracks['audio'][0]
    else:
        print("ERROR: bad audio tracks:", tracks['audio'])
        exit()

    # smash together only the video and audio tracks, and the subs file
    cmd = [
        "mkvmerge",
        "--quiet",
        "--no-subtitles", "--no-buttons", "--no-track-tags", "--no-chapters",
        "--video-tracks", video_track,
        "--audio-tracks", audio_track,
        moviefilepath,
        srtfilepath,
        "--output", result_filepath,
    ]
    subprocess.run(cmd)


USAGE = """
  Usage: merge_video_subs.py
        [--name-from-dir] [--name=NAME] [--audio-track=N]
        file.movie [-sub] file.srt

    Valid extensions for the movie: {extensions}

    The -sub is totally ignored, it's just to reuse mplayer's previous command.

    If --name-from-dir, the result name will be taken from current dir. If --name, the result
    name will be what is specified. Both are excluyent.
""".format(extensions=", ".join(VIDEO_EXTS))

if __name__ == '__main__':
    parser = argparse.ArgumentParser(usage=USAGE)
    parser.add_argument('-sub', action='store_true')  # don't care about this, just for easier cmd
    parser.add_argument('--audio-track', dest='audio_track', action='store')

    mexc = parser.add_mutually_exclusive_group()
    mexc.add_argument('--name-from-dir', dest='name_from_dir', action='store_true')
    mexc.add_argument('--name', dest='result_name', action='store')

    parser.add_argument('source_movie')
    parser.add_argument('source_subs')
    args = parser.parse_args()

    if args.name_from_dir:
        current_dir = os.path.basename(os.path.abspath(os.path.curdir))
        result_filepath = current_dir + ".mkv"
        if result_filepath == args.source_movie:
            result_filepath = current_dir + "--merged.mkv"
    elif args.result_name:
        result_filepath = args.result_name
    else:
        result_filepath = get_output_name(args.source_movie)

    main(args.source_movie, args.source_subs, result_filepath, args.audio_track)
